from .._wpiutil.sync import (
    createEvent,
    createSemaphore,
    createSignalObject,
    destroyEvent,
    destroySemaphore,
    destroySignalObject,
    releaseSemaphore,
    resetEvent,
    resetSignalObject,
    setEvent,
    setSignalObject,
    waitForObject,
    waitForObjects,
)

__all__ = [
    "createEvent",
    "createSemaphore",
    "createSignalObject",
    "destroyEvent",
    "destroySemaphore",
    "destroySignalObject",
    "releaseSemaphore",
    "resetEvent",
    "resetSignalObject",
    "setEvent",
    "setSignalObject",
    "waitForObject",
    "waitForObjects",
]



from wpiutil_test import module


def test_smallset_load():
    assert module.load_smallset_int({1, 2, 3, 4}) == {1, 2, 3, 4}


def test_smallsetbool_load():
    assert module.load_smallset_int({True, True, False, True}) == {
        True,
        True,
        False,
        True,
    }


def test_smallset_cast():
    assert module.cast_smallset() == {1, 2, 3, 4}



import typing
import wpiutil
from wpiutil_test import module


class MySendable(wpiutil.Sendable):
    def __init__(self):
        super().__init__()
        wpiutil.SendableRegistry.add(self, "Test", 1)
        self.value = 0

    def initSendable(self, builder: wpiutil.SendableBuilder):
        builder.addDoubleProperty("key", self._get, self._set)

    def _set(self, value: float):
        self.value = value

    def _get(self) -> float:
        return self.value


def test_custom_sendable():
    ms = MySendable()

    uid = wpiutil.SendableRegistry.getUniqueId(ms)
    keys = {}

    module.publish(uid, keys)
    assert ms.value == 0

    getter, setter = keys["key"]
    assert getter() == 0
    setter(1)
    assert getter() == 1
    assert ms.value == 1



# autogenerated by 'robotpy-build create-imports wpiutil.log wpiutil._wpiutil.log'
from .._wpiutil.log import (
    BooleanArrayLogEntry,
    BooleanLogEntry,
    DataLog,
    DataLogEntry,
    DataLogReader,
    DataLogRecord,
    DoubleArrayLogEntry,
    DoubleLogEntry,
    FloatArrayLogEntry,
    FloatLogEntry,
    IntegerArrayLogEntry,
    IntegerLogEntry,
    MetadataRecordData,
    RawLogEntry,
    StartRecordData,
    StringArrayLogEntry,
    StringLogEntry,
)

__all__ = [
    "BooleanArrayLogEntry",
    "BooleanLogEntry",
    "DataLog",
    "DataLogEntry",
    "DataLogReader",
    "DataLogRecord",
    "DoubleArrayLogEntry",
    "DoubleLogEntry",
    "FloatArrayLogEntry",
    "FloatLogEntry",
    "IntegerArrayLogEntry",
    "IntegerLogEntry",
    "MetadataRecordData",
    "RawLogEntry",
    "StartRecordData",
    "StringArrayLogEntry",
    "StringLogEntry",
]



from wpiutil_test import module


def test_load_array_int():
    assert module.load_array_int((1, 2, 3, 4)) == (1, 2, 3, 4)
    assert module.load_array_int([1, 2, 3, 4]) == (1, 2, 3, 4)


def test_load_array_annotation():
    assert (
        module.load_array_int.__doc__
        == "load_array_int(arg0: Tuple[int, int, int, int]) -> Tuple[int, int, int, int]\n"
    )
    assert (
        module.load_array_int1.__doc__
        == "load_array_int1(arg0: Tuple[int]) -> Tuple[int]\n"
    )



from traceback import extract_stack, format_list
from ._wpiutil import getStackTraceDefault, _setup_stack_trace_hook
from os.path import join

_start_py = join("wpilib", "_impl", "start.py")


def _stack_trace_hook(offset: int) -> str:
    # note: this implementation ignores offset because it's not
    # actually meaningful when crossing the python/C++ boundary

    stack = extract_stack()[:-1]
    if not stack:
        return "\tat <no python frames>\n" + getStackTraceDefault(offset)

    # filter out any frames before start.py (except for one of them) to
    # make stack frames more useful for users
    for i in range(len(stack) - 1, 0, -1):
        if stack[i].filename.endswith(_start_py):
            stack = stack[i:]
            break

    trace = format_list(stack)
    return "\n".join(trace)


_setup_stack_trace_hook(_stack_trace_hook)



#!/usr/bin/env python3
#
# Copyright (c) FIRST and other WPILib contributors.
# Open Source Software; you can modify and/or share it under the terms of
# the WPILib BSD license file in the root directory of this project.

import argparse
import datetime

from wpiutil.log import DataLogReader

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("infile")
    args = parser.parse_args()

    reader = DataLogReader(args.infile)

    entries = {}
    for record in reader:
        timestamp = record.getTimestamp() / 1000000
        if record.isStart():
            try:
                data = record.getStartData()
                print(f"{data} [{timestamp}]")
                if data.entry in entries:
                    print("...DUPLICATE entry ID, overriding")
                entries[data.entry] = data
            except TypeError as e:
                print("Start(INVALID)")
        elif record.isFinish():
            try:
                entry = record.getFinishEntry()
                print(f"Finish({entry}) [{timestamp}]")
                if entry not in entries:
                    print("...ID not found")
                else:
                    del entries[entry]
            except TypeError as e:
                print("Finish(INVALID)")
        elif record.isSetMetadata():
            try:
                data = record.getSetMetadataData()
                print(f"{data} [{timestamp}]")
                if data.entry not in entries:
                    print("...ID not found")
            except TypeError as e:
                print("SetMetadata(INVALID)")
        elif record.isControl():
            print("Unrecognized control record")
        else:
            print(f"Data({record.getEntry()}, size={record.getSize()}) ", end="")
            entry = entries.get(record.getEntry(), None)
            if entry is None:
                print("<ID not found>")
                continue
            print(f"<name='{entry.name}', type='{entry.type}'> [{timestamp}]")

            try:
                # handle systemTime specially
                if entry.name == "systemTime" and entry.type == "int64":
                    dt = datetime.fromtimestamp(record.getInteger() / 1000000)
                    print("  {:%Y-%m-%d %H:%M:%S.%f}".format(dt))
                    continue

                if entry.type == "double":
                    print(f"  {record.getDouble()}")
                elif entry.type == "int64":
                    print(f"  {record.getInteger()}")
                elif entry.type == "string" or entry.type == "json":
                    print(f"  '{record.getString()}'")
                elif entry.type == "boolean":
                    print(f"  {record.getBoolean()}")
                elif entry.type == "boolean[]":
                    arr = record.getBooleanArray()
                    print(f"  {arr}")
                elif entry.type == "double[]":
                    arr = record.getDoubleArray()
                    print(f"  {arr}")
                elif entry.type == "float[]":
                    arr = record.getFloatArray()
                    print(f"  {arr}")
                elif entry.type == "int64[]":
                    arr = record.getIntegerArray()
                    print(f"  {arr}")
                elif entry.type == "string[]":
                    arr = record.getStringArray()
                    print(f"  {arr}")
                elif entry.type == "raw":
                    print(f"  {record.getRaw()}")
            except TypeError as e:
                print("  invalid", e)



from wpiutil_test import module


def test_smallvec_load():
    assert module.load_smallvec_int([1, 2, 3, 4]) == [1, 2, 3, 4]


def test_smallvecbool_load():
    assert module.load_smallvec_int([True, True, False, True]) == [
        True,
        True,
        False,
        True,
    ]


def test_smallvec_cast():
    assert module.cast_smallvec() == [1, 2, 3, 4]


def test_smallvecimpl_load():
    assert module.load_smallvecimpl_int([1, 2, 3, 4]) == [1, 2, 3, 4]



#!/usr/bin/env python3

import argparse
import pathlib

from wpiutil.log import DataLog, BooleanLogEntry, StringArrayLogEntry, RawLogEntry


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("out", type=pathlib.Path)
    args = parser.parse_args()

    if args.out.is_dir():
        datalog = DataLog(str(args.out))
    else:
        datalog = DataLog(str(args.out.parent), args.out.name)

    bools = BooleanLogEntry(datalog, "/bools")
    bools.append(True)
    bools.append(False)

    strings = StringArrayLogEntry(datalog, "/strings")
    strings.append(["a", "b", "c"])
    strings.append(["d", "e", "f"])

    raw = RawLogEntry(datalog, "/raws")
    raw.append(b"\x01\x02\x03")
    raw.append(b"\x04\x05\x06")



from . import _init_wpiutil

# autogenerated by 'robotpy-build create-imports wpiutil wpiutil._wpiutil'
from ._wpiutil import (
    ControlRecordType,
    Sendable,
    SendableBuilder,
    SendableRegistry,
    getStackTrace,
    getStackTraceDefault,
    # log,
)

__all__ = [
    "ControlRecordType",
    "Sendable",
    "SendableBuilder",
    "SendableRegistry",
    "getStackTrace",
    "getStackTraceDefault",
    # "log",
]

# Imported for side effects only
from . import _stacktrace



from wpiutil_test.module import (
    cast_json_arg,
    cast_json_val,
    max_int64,
    min_int64,
    max_uint64,
)
import pytest
import math


def test_json_invalid():
    with pytest.raises(TypeError):
        cast_json_val(lambda: object())


def test_json_none():
    assert cast_json_arg(None) == None


def test_json_bool():
    assert cast_json_arg(True) == True
    assert cast_json_arg(False) == False


def test_json_int():
    assert cast_json_arg(36) == 36

    assert cast_json_arg(min_int64) == min_int64
    with pytest.raises(ValueError):
        cast_json_arg(min_int64 - 1)

    assert cast_json_arg(max_int64) == max_int64
    assert cast_json_arg(max_uint64) == max_uint64
    with pytest.raises(ValueError):
        cast_json_arg(max_uint64 + 1)


def test_json_float():
    assert cast_json_arg(36.37) == 36.37
    assert cast_json_arg(math.inf) == math.inf
    assert math.isnan(cast_json_arg(math.nan))


def test_json_string():
    assert cast_json_arg("hi") == "hi"


def test_json_list():
    v = [36, "hello", False]
    assert cast_json_arg(v) == v

    assert cast_json_arg([]) == []

    tv = (36, "hello", False)
    assert cast_json_arg(tv) == v


def test_json_dict():
    d = {"number": 1234, "hello": "world"}
    assert cast_json_arg(d) == d

    assert cast_json_arg({}) == {}

    assert cast_json_arg({1: 2}) == {"1": 2}
    assert cast_json_arg({None: 2}) == {"None": 2}
    assert cast_json_arg({1.2: 2}) == {"1.2": 2}
    assert cast_json_arg({False: 2}) == {"False": 2}

    with pytest.raises(TypeError):
        cast_json_arg({object(): 2})



import pytest
from wpiutil_test import module
import array


def test_span_load_int():
    assert module.load_span_int([1, 2, 3, 4]) == [1, 2, 3, 4]


def test_span_load_int():
    assert module.load_span_int([1, 2, 3]) == [1, 2, 3]


def test_span_load_bool():
    assert module.load_span_bool([True, False, True]) == [True, False, True]


def test_span_load_string():
    assert module.load_span_string(["a", "b", "c"]) == ["a", "b", "c"]


def test_span_load_string_const():
    assert module.load_span_string_const(["a", "b", "c"]) == ["a", "b", "c"]


def test_span_load_stringview():
    assert module.load_span_string_view(["a", "b", "c"]) == ["a", "b", "c"]


def test_span_load_vector():
    assert module.load_span_vector([["a"], ["b"], ["c"]]) == [["a"], ["b"], ["c"]]


def test_span_load_buffer_bytes():
    assert module.load_span_bytes(b"abc") == b"abc"


def test_span_modify_buffer_bytes():
    b = b"abc"
    with pytest.raises(BufferError):
        module.modify_span_buffer(b)


def test_span_load_buffer_bytearray():
    assert module.load_span_bytes(bytearray([1, 2, 3])) == b"\x01\x02\x03"


def test_span_modify_buffer_bytearray():
    b = bytearray([1, 2, 3])
    module.modify_span_buffer(b)
    assert b == bytearray([4, 2, 3])


def test_span_load_buffer_array():
    a = array.array("l")
    a.append(1)
    a2 = array.array("l")
    a2.frombytes(module.load_span_bytes(a))
    assert len(a2) == 1
    assert a2[0] == 1


def test_span_cast():
    assert module.cast_span() == [1, 2, 3]


def test_string_span():
    assert module.cast_string_span() == ["hi", "there"]


def test_fixed_double_span():
    assert module.load_span_fixed_double([1, 2, 3]) == (1, 2, 3)

    with pytest.raises(TypeError):
        assert module.load_span_fixed_double([1, 2, 3, 4])



from wpiutil_test import module


def test_stringmap_load():
    assert module.load_stringmap_int({"one": 11, "two": 22, "three": 33}) == {
        "one": 11,
        "two": 22,
        "three": 33,
    }


def test_stringmap_cast():
    assert module.cast_stringmap() == {"one": 1, "two": 2}
